/*   AM-STD for ICP-PCI Kernel module
 *   free public release
 *
 *   Modified by: Rolando E. Cantarutti email: rcantarutti@noao.edu
 *                Cerro Tololo Interamerican Observatory
 *                AURA INC.
 *
 *      referring to: AM-CLR developed by the following people
 *
 *   Werner Jaeger        email: e9026603@student.tuwien.ac.at
 *   Wilfried Kubinger    email: wk@flexaut.tuwien.ac.at
 *
 *   Institute of Flexible Automation
 *   University of Technology Vienna
 *   Austria, Europe
 *
 *   amcmpReg.h  v1.0  1999/09/09
 */

/* Change History:

 */

#ifndef _AMCMPREG_H
#define _AMCMPREG_H

#if defined(__KERNEL__) || defined(__CONFIG__)

/* AM-STD Hardware Reference Manual; Imaging Technology Inc.*/
/* Document Number 47-H44007-02; December 1997 */

/* R-O   Read-Only Register */
/* W-O   Write-Only Register */
/* R/W   Read/Write Register */
/* R/W1C Clear on Write One to Register */
/* R/W0C Clear on Write Zero to Register */

/* AM-STD Register Set (16bit access only), pages 4-1 to 4-31 */
/* ********************************************************* */

/* 
 * Prototypes
 */

/* read handler */

u_char cmp_get_rev (u_char);
u_char cmp_get_brdid (u_char);
u_short cmp_get_vactive (u_char);
u_short cmp_get_hactive (u_char);
u_short cmp_get_vscale (u_char);

/* write handler */

int cmp_set_camera (u_char, u_char);
int cmp_set_status (u_char board, u_char value);
int cmp_set_iform (u_char board, u_char value);
int cmp_set_tdec (u_char board, u_char value);
int cmp_set_hscale (u_char board, u_short value);
int cmp_set_bright (u_char board, u_char value);
int cmp_set_control (u_char board, u_char value);
int cmp_set_contrast (u_char board, u_short value);
int cmp_set_sat_u (u_char board, u_short value);
int cmp_set_sat_v (u_char board, u_short value);
int cmp_set_hue (u_char board, u_char value);
int cmp_set_scloop (u_char board, u_char value);
int cmp_set_oform (u_char board, u_char value);
int cmp_set_vscal_hi (u_char board, u_char value);
int cmp_set_vscale (u_char board, u_short value);
int cmp_set_adly (u_char board, u_char value);
int cmp_set_bdly (u_char board, u_char value);
int cmp_set_adc (u_char board, u_char value);
int cmp_set_vtc (u_char board, u_char value);
int cmp_set_vdelay (u_char, u_short);
int cmp_set_vactive (u_char, u_short);
int cmp_set_hdelay (u_char, u_short);
int cmp_set_hactive (u_char, u_short);

/* init handler */

int cmp_init_mxctl (u_char);
int cmp_init_scl (u_char);
int cmp_init_sda (u_char);
int cmp_init_nptrig (u_char);
int cmp_init_vdecregs (u_char);


/* misc handler */

void cmp_dump_regs (u_char);

/* end of prototypes */

#define CMP_REG_SIZE		6	/* size of register set */

#define CMP_NO_MASK		0xFFFF	/* no checking */
#define CMP_NO_MIN		0x0000
#define CMP_NO_MAX		0xFFFF


/*    u_short      modid         Module Identification */
/* -------------------------------------------------- */

#define CMP_MODID_OFFSET         0x00

/*    u_short      rev:3         Revision R-O */

#define CMP_REV_MASK             0x0007
#define CMP_REV_SHIFT            0

/*    u_short      brdid:5       Board ID R-O */

#define CMP_BRDID_MASK		 0x00F8
#define CMP_BRDID_SHIFT		 3


/*    u_short      mxctl         Ouput Multiplexer Control R/W */
/* ---------------------------------------------------------- */

#define CMP_MXCTL_OFFSET	0x01
#define CMP_MXCTL_INIT		0x0004


/*    u_short      scl           Serial Port Clock Control */
/* ------------------------------------------------------ */

#define CMP_SCL_OFFSET		0x02
#define CMP_SCL_INIT		0x0001
				/* Serial Clock Disable */

/*    u_short      scl0:1        Serial Clock R/W */

#define CMP_SCL0_MASK		0x0001
#define CMP_SCL0_SHIFT		0
#define CMP_SCL0_LOW		0
#define CMP_SCL0_HIGH		1

/*    u_short      sclen:1       Serial Clock Enable R/W */

#define CMP_SCLEN_MASK		0x0002
#define CMP_SCLEN_SHIFT		1
#define CMP_SCLEN_DISABLE	0
#define CMP_SCLEN_ENABLE	1


/*    u_short      sda           Serial Port Data */
/* --------------------------------------------- */

#define CMP_SDA_OFFSET          0x03
#define CMP_SDA_INIT		0x0001
				/* Serial Data Disable */

/*    u_short      sda0:1        Serial Data W-O */

#define CMP_SDA0_MASK           0x0001
#define CMP_SDA0_SHIFT          0
#define CMP_SDA0_LOW            0
#define CMP_SDA0_HIGH           1

/*    u_short      sdaen:1	Serial Data Enable R/W */

#define CMP_SDAEN_MASK 		0x0002
#define CMP_SDAEN_SHIFT         1
#define CMP_SDAEN_DISABLE       0
#define CMP_SDAEN_ENABLE        1

/*    u_short      sdarbk:1      Serial Data Read Back R-O */

#define CMP_SDARBK_MASK		0x0004
#define CMP_SDARBK_SHIFT        2

/*    u_short      nptrig        Trigger Control R/W */
/* ------------------------------------------------ */

#define CMP_NPTRIG_OFFSET	0x04
#define CMP_NPTRIG_INIT		0x0000

/*    u_short      ntrgen        Trigger Enable R/W */
/* ----------------------------------------------- */

#define CMP_NTRGEN_MASK		0x0001
#define CMP_NTRGEN_SHIFT	0
#define CMP_NTRGEN_DISABLE      0
#define CMP_NTRGEN_ENABLE	1


#define CMP_BT829A_REG_SIZE	32

/*    BT829A I2C Base Address and R/W bit	*/
/* -------------------------------------------- */

#define CMP_BT829A_W_ADDRESS	0x88
#define CMP_BT829A_R_ADDRESS	0x89

/*    u_char       status	Decoder Status R/W */
/* ----------------------------------------------- */

#define CMP_BT829A_STATUS_ADDRESS	0x00
#define CMP_BT829A_STATUS_INIT		0x00

/*    u_char       cof           Chroma ADC Overflow Status R/W */

#define CMP_BT829A_COF_MASK             0x01
#define CMP_BT829A_COF_SHIFT            0
#define CHROMA_OVERFLOW                 0x00
#define CHROMA_NORMAL                   0x01

/*    u_char       lof           Luma ADC Overflow Status R/W */

#define CMP_BT829A_LOF_MASK             0x02
#define CMP_BT829A_LOF_SHIFT            1
#define LUMA_OVERFLOW                   0x00
#define LUMA_NORMAL                     0x02

/*    u_char       csel          Crystal Select Status R/W */

#define CMP_BT829A_CSEL_MASK            0x08
#define CMP_BT829A_CSEL_SHIFT           3
#define NTSC_PALM                       0x00
#define PAL_SECAM                       0x08

/*    u_char       numl          Number of Lines Status R/W */

#define CMP_BT829A_NUML_MASK            0x10
#define CMP_BT829A_NUML_SHIFT           4
#define LINE_525                        0x00
#define LINE_625                        0x10

/*    u_char       field         Field Status R/W */

#define CMP_BT829A_FIELD_MASK           0x20
#define CMP_BT829A_FIELD_SHIFT          5
#define ODD_FIELD                       0x00
#define EVEN_FIELD                      0x20

/*    u_char       hloc          Horizontal Lock Status */

#define CMP_BT829A_HLOC_MASK            0x40
#define CMP_BT829A_HLOC_SHIFT           6
#define NOT_IN_HLOCK                    0x00
#define DEC_IN_HLOCK                    0x40

/*    u_char       pres          Video Present Status */

#define CMP_BT829A_PRES_MASK            0x80
#define CMP_BT829A_PRES_SHIFT           7
#define VIDEO_NOT_PRESENT               0x00
#define VIDEO_PRESENT                   0x80

/*    u_char       iform         Input Format R/W */
/* --------------------------------------------- */

#define CMP_BT829A_IFORM_ADDRESS	0x01
#define CMP_BT829A_IFORM_INIT		0x4A

/*    u_char       format        Input Format Select R/W */

#define CMP_BT829A_FORMAT_MASK          0x07
#define CMP_BT829A_FORMAT_SHIFT         0
#define AUTO_FORMAT                     0x00
#define NTSC_M                          0x01
#define NTSC_NO_PEDESTAL                0x02
#define PAL_BDGHI                       0x03
#define PAL_M                           0x04
#define PAL_N                           0x05
#define SECAM                           0x06

/*    u_char       xtsel         Crystal Select R/W */

#define CMP_BT829A_XTSEL_MASK           0x18
#define CMP_BT829A_XTSEL_SHIFT          3
#define NTSC_FREQ                       0x08
#define PAL_SECAM_FREQ                  0x10
#define AUTO_FREQ                       0x18

/*    u_char       muxsel	Input MUX Select R/W */

#define CMP_BT829A_MUXSEL_MASK		0x60
#define CMP_BT829A_MUXSEL_SHIFT		5
#define CMP_CAMERA_0			0x2
#define CMP_CAMERA_1			0x3
#define CMP_CAMERA_2			0x1
#define CMP_CAMERA_3			0x0

/*    u_char       tdec		Temporal Decimation R/W */
/* ---------------------------------------------------- */

#define CMP_BT829A_TDEC_ADDRESS         0x02
#define CMP_BT829A_TDEC_INIT            0x00

/*    u_char       decrat        Decimiation Rate R/W */

#define CMP_BT829A_DECRAT_MASK          0x3F
#define CMP_BT829A_DECRAT_SHIFT         0

/*    u_char       fldalign      Field Align R/W */

#define CMP_BT829A_FLDALIGN_MASK        0x40
#define CMP_BT829A_FLDALIGN_SHIFT       6
#define DEC_ON_ODD_FLD                  0x00
#define DEC_ON_EVEN_FLD                 0x40

/*    u_char       decfld        Decimate Fields/Frames R/W */

#define CMP_BT829A_DECFLD_MASK          0x80
#define CMP_BT829A_DECFLD_SHIFT         7
#define DEC_FRAMES                      0x00
#define DEC_FIELDS                      0x80

/*    u_char       crop		MSB Cropping R/W */
/* --------------------------------------------- */

#define CMP_BT829A_CROP_ADDRESS		0x03
#define CMP_BT829A_CROP_INIT		0x12

/*    u_char       hact          Horizontal Active High R/W */

#define CMP_BT829A_HACT_MASK		0x03
#define CMP_BT829A_HACT_SHIFT		0

/*    u_char       hdly          Horizontal Delay High R/W */

#define CMP_BT829A_HDLY_MASK		0x0C
#define CMP_BT829A_HDLY_SHIFT		2

/*    u_char       vact          Vertical Active High R/W */

#define CMP_BT829A_VACT_MASK		0x30
#define CMP_BT829A_VACT_SHIFT		4

/*    u_char       vdelay        Vertical Delay High R/W */

#define CMP_BT829A_VDLY_MASK		0xC0
#define CMPBT829A_VDLY_SHIFT		6

/*    u_char       vdly_lo       Vertical Delay Low R/W */
/* ---------------------------------------------------- */

#define CMP_BT829A_VDLY_LO_ADDRESS	0x04
#define CMP_BT829A_VDLY_LO_INIT		0x16

/*    u_char       vact_lo       Vertical Active Low R/W */
/* ---------------------------------------------------- */

#define CMP_BT829A_VACT_LO_ADDRESS	0x05
#define CMP_BT829A_VACT_LO_INIT		0xE0

/*    u_char       hdly_lo	Horizontal Delay Low R/W */
/* ----------------------------------------------------- */

#define CMP_BT829A_HDLY_LO_ADDRESS	0x06
#define CMP_BT829A_HDLY_LO_INIT		0x72

/*    u_char       hact_lo	Horizontal Active Low R/W */
/* ------------------------------------------------------ */

#define CMP_BT829A_HACT_LO_ADDRESS	0x07
#define CMP_BT829A_HACT_LO_INIT		0x80

/*    u_char       hscal_hi      Horizontal Scale High R/W */
/* ------------------------------------------------------ */

#define CMP_BT829A_HSCAL_HI_ADDRESS	0x08
#define CMP_BT829A_HSCAL_HI_INIT	0x02

/*    u_char       hscal_lo      Horizontal Scale Low R/W */
/* ----------------------------------------------------- */

#define CMP_BT829A_HSCAL_LO_ADDRESS	0x09
#define CMP_BT829A_HSCAL_LO_INIT	0xAA

/*    u_char       bright        Brightness Control R/W */
/* --------------------------------------------------- */

#define CMP_BT829A_BRIGHT_ADDRESS	0x0A
#define CMP_BT829A_BRIGHT_INIT		0x00

/*    u_char       control       Miscellaneous Control R/W */
/* ------------------------------------------------------ */

#define CMP_BT829A_CONTROL_ADDRESS      0x0B
#define CMP_BT829A_CONTROL_INIT         0x20

/*    u_char       sat_v_msb     Saturation V Gain msb R/W */

#define CMP_BT829A_SAT_V_MSB_MASK       0x01
#define CMP_BT829A_SAT_V_MSB_SHIFT      0

/*    u_char       sat_u_msb     Saturation U Gain msb R/W */

#define CMP_BT829A_SAT_U_MSB_MASK       0x02
#define CMP_BT829A_SAT_U_MSB_SHIFT      1

/*    u_char       con_msb       Luma Gain msb R/W */

#define CMP_BT829A_CON_MSB_MASK         0x04
#define CMP_BT829A_CON_MSB_SHIFT        2

/*    u_char       cbsense       Color Difference Ordering R/W */

#define CMP_BT829A_CBSENSE_MASK         0x10
#define CMP_BT829A_CBSENSE_SHIFT        4
#define NORMAL_ORDER                    0x00
#define INVERT_ORDER                    0x10

/*    u_char       ldec          Luma Decimation Control R/W */

#define CMP_BT829A_LDEC_MASK            0x20
#define CMP_BT829A_LDEC_SHIFT           5
#define ENABLE_LUMA_DEC                 0x00
#define DISABLE_LUMA_DEC                0x20

/*    u_char       comp          Composite Video Select R/W */

#define CMP_BT829A_COMP_MASK            0x40
#define CMP_BT829A_COMP_SHIFT           6
#define COMPOSITE_VIDEO                 0x00
#define YC_COMPONENT_VIDEO              0x40

/*    u_char       lnotch        Luma Notch Filter Control R/W */

#define CMP_BT829A_LNOTCH_MASK          0x80
#define CMP_BT829A_LNOTCH_SHIFT         7
#define ENABLE_NOTCH_FILTER             0x00
#define DISABLE_NOTCH_FILTER            0x80

/*    u_char       con_lo        Luma Gain Low R/W */
/* ---------------------------------------------- */

#define CMP_BT829A_CON_LO_ADDRESS       0x0C
#define CMP_BT829A_CON_LO_INIT          0xD8

/*    u_char       sat_u_lo      Chroma U Gain Low R/W */
/* -------------------------------------------------- */

#define CMP_BT829A_SAT_U_LO_ADDRESS     0x0D
#define CMP_BT829A_SAT_U_LO_INIT        0xFE

/*    u_char       sat_v_lo      Chroma V Gain Low R/W */
/* -------------------------------------------------- */

#define CMP_BT829A_SAT_V_LO_ADDRESS     0x0E
#define CMP_BT829A_SAT_V_LO_INIT        0xB4

/*    u_char       hue           Hue Adjust R/W */
/* ------------------------------------------- */

#define CMP_BT829A_HUE_ADDRESS          0x0F
#define CMP_BT829A_HUE_INIT             0x00

/*    u_char       scloop        SC Loop control R/W */
/* ------------------------------------------------ */

#define CMP_BT829A_SCLOOP_ADDRESS       0x10
#define CMP_BT829A_SCLOOP_INIT          0x00

/*    u_char       hfilt         Horizontal Filter Select R/W */

#define CMP_BT829A_HFILT_MASK           0x18
#define CMP_BT829A_HFILT_SHIFT          3
#define AUTOMATIC_FORMAT                0x00
#define CIF_SCALE                       0x08
#define QCIF_SCALE                      0x10
#define ICON_SCALE                      0x18

/*    u_char       ckill         Color Kill Enable R/W */

#define CMP_BT829A_CKILL_MASK           0x20
#define CMP_BT829A_CKILL_SHIFT          5
#define DISABLE_LOW_DETECTION           0x00
#define ENABLE_LOW_DETECTION            0x20

/*    u_char       cagc          Chroma Automatic Gain Enable R/W */

#define CMP_BT829A_CAGC_MASK            0x40
#define CMP_BT829A_CAGC_SHIFT           6
#define DISABLE_CAGC                    0x00
#define ENABLE_CAGC                     0x40

/*    u_char       oform         Output Format R/W */
/* ---------------------------------------------- */

#define CMP_BT829A_OFORM_ADDRESS        0x12
#define CMP_BT829A_OFORM_INIT           0x06

/*    u_char       len           Output Data Format R/W */

#define CMP_BT829A_LEN_MASK             0x04
#define CMP_BT829A_LEN_SHIFT            2
#define OUTPUT_16                       0x04

/*    u_char       core          Luma Coring Select R/W */

#define CMP_BT829A_CORE_MASK            0x60
#define CMP_BT829A_CORE_SHIFT           5
#define THRESHOLD_0                     0x00
#define THRESHOLD_8                     0x20
#define THRESHOLD_16                    0x40
#define THRESHOLD_32                    0x60

/*    u_char       range         Luma Output Range R/W */

#define CMP_BT829A_RANGE_MASK           0x80
#define CMP_BT829A_RANGE_SHIFT          7
#define NORMAL_RANGE                    0x00
#define FULL_RANGE                      0x80

/*    u_char       vscal_hi      Vertical Scaling High R/W */
/* ------------------------------------------------------ */

#define CMP_BT829A_VSCAL_HI_ADDRESS	0x13
#define CMP_BT829A_VSCAL_HI_INIT	0x60

/*    u_short	  vscal		Vertical Scale R/W */

#define CMP_BT829A_VSCAL_MASK		0x1F
#define CMP_BT829A_VSCAL_SHIFT		0

/*    u_char       int           Interlace Select R/W */

#define CMP_BT829A_INT_MASK		0x20
#define CMP_BT829A_INT_SHIFT		5
#define NON_INTERLACE_VS                0x00
#define INTERLACE_VS                    0x20

/*    u_char       ccomb         Chroma Comb Filter Enable R/W */

#define CMP_BT829A_CCOMB_MASK           0x40
#define CMP_BT829A_CCOMB_SHIFT          6
#define DISABLE_CCOMB                   0x00
#define ENABLE_CCOMB                    0x40

/*    u_char       ycomb         Luma Comb Filter Enable R/W */ 

#define CMP_BT829A_YCOMB_MASK           0x80
#define CMP_BT829A_YCOMB_SHIFT          7
#define DISABLE_YCOMB                   0x00
#define ENABLE_YCOMB                    0x80


/*    u_char       vscal_lo      Vertical Scaling Low R/W */
/* ------------------------------------------------------ */

#define CMP_BT829A_VSCAL_LO_ADDRESS     0x14
#define CMP_BT829A_VSCAL_LO_INIT        0x00

/*    u_char       idcode        ID Code R-O */
/* ---------------------------------------- */

#define CMP_BT829A_IDCODE_ADDRESS       0x17

/*    u_char       adly          AGC Delay R/W */
/* ------------------------------------------ */

#define CMP_BT829A_ADLY_ADDRESS         0x18
#define CMP_BT829A_ADLY_INIT            0x68

/*    u_char       bdly          Burst Gate Delay R/W */
/* ------------------------------------------------- */

#define CMP_BT829A_BDLY_ADDRESS         0x19
#define CMP_BT829A_BDLY_INIT            0x5D 

/*    u_char       adc           ADC Interface R/W */
/* ---------------------------------------------- */

#define CMP_BT829A_ADC_ADDRESS          0x1A
#define CMP_BT829A_ADC_INIT             0x82

/*    u_char       csleep        Chroma Sleep R/W */

#define CMP_BT829A_CSLEEP_MASK          0x02
#define NORMAL_CHROMA                   0x00
#define SLEEP_CHROMA                    0x02

/*    u_char       ysleep        Luma Sleep R/W */

#define CMP_BT829A_YSLEEP_MASK          0x04
#define NORMAL_LUMA                     0x00
#define SLEEP_LUMA                      0x04

/*    u_char       clksleep      Clock Sleep R/W */

#define CMP_BT829A_CLKSLEEP_MASK        0x08
#define NORMAL_CLOCK                    0x00
#define SLEEP_CLOCK                     0x08

/*    u_char       synct         Sync Threshold R/W */

#define CMP_BT829A_SYNCT_MASK           0x20
#define HIGH_SYNC                       0x00
#define LOW_SYNC                        0x20

/*    u_char       vtc           Video Timing Control R/W */
/* ----------------------------------------------------- */

#define CMP_BT829A_VTC_ADDRESS		0x1B
#define CMP_BT829A_VTC_INIT		0x40

/*    u_char       vfilt         Vertical Scaling Filter R/W */

#define CMP_BT829A_VFILT_MASK           0x03
#define FILTER_2_TAP                    0x00
#define FILTER_3_TAP                    0x01
#define FILTER_4_TAP                    0x02
#define FILTER_5_TAP                    0x03

/*    u_char       acfmt         Active Format R/W */

#define CMP_BT829A_ACFMT_MASK           0x40
#define HORIZONTAL_ACTIVE               0x40

/*    u_char       hsfmt         Horizontal Reset Format R/W */

#define CMP_BT829A_HSFMT_MASK           0x80
#define HORIZONTAL_RESET                0x00

/*    u_char       sreset        Software Reset R/W */
/* ----------------------------------------------- */

#define CMP_BT829A_SRESET_ADDRESS	0x1F
#define CMP_BT829A_SRESET_RESET		0


#endif /* __KERNEL__ || __CONFIG__ */

#endif /* _AMVSREG_H */

