/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 *
 * $Id: icpciInt.h,v 5.1.2.3 1999/07/21 08:42:38 matthias Exp $
 *
 */

#ifndef _ICPCIINT_H
#define _ICPCIINT_H

#if defined(linux) && defined(__KERNEL__)
/*
 * Includes:
 */

/*
 * Defines
 */

#define ROUND_DOWNTO_PAGE(addr) \
	((unsigned long)(addr) & ~(PAGE_SIZE - 1))
     
#define ROUND_UPTO_PAGE(addr) \
	(((unsigned long)(addr) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1))

#ifdef ICP_WAIT_LOOP
#define	ICP_DELAY(x) \
	do { \
             for (icp.cap[board]->dummy_cnt = 0; \
		  icp.cap[board]->dummy_cnt < x; \
		  icp.cap[board]->dummy_cnt++) \
	            *icp.cap[board]->dummy_ptr = \
		      icp.cap[board]->dummy_cnt; \
	} while (0);  
#else
#define	ICP_DELAY(x)	icp_udelay (x)
#endif

#define	ICP_AMCC_READ	0	/* current op is read op */
#define ICP_AMCC_WRITE	1	/* current op is write op */
#define	ICP_AMCC_EVEN	0	/* latch status even read/write */
#define ICP_AMCC_ODD	1	/* latch status odd read/write */

#define MEM_MAP_SIZE	0x00200000	/* Frame Grabber on Board Memory
					   Size 2 Mb default */
#define MOD_MAP_SIZE	(2 * PAGE_SIZE)	/* 8k Acquisition Module Register
					   Space (2 Pages)*/
     
#define ICP_MODID_OFFST	0x00	/* where to find the modid in reg space */
#define ICP_TYPE_MASK	0x00F8	/* decoded modid bits */
#define ICP_TYPE_SHIFT	3	/* shifts for a useful value */

#define ICP_MEMSZ_MASK	0x80	/* on board memory size is decoded */
#define ICP_MEMSZ_SHIFT	7	/* in revision id register */
#define ICP_LEVEL_MASK	0x80	/* revision 1 level is decoded */
#define ICP_LEVEL_SHIFT	7	/* in status register */

#define BLOCK_CLEAR_SIZE	(64 << 10) /* clear 64k of on-board memory
					      without a schedule */
     
#define ICP_CLEAR_BLACK	0	/* fill frame with black pixels */
#define ICP_CLEAR_GREY	1	/* fill frame with grey pixels */
#define ICP_CLEAR_WHITE	2	/* fill frame with white pixels */
#define ICP_ALL_BUFFERS	4	/* fill all frames */
     
/* interrupt activation */
#define PV_SOTINT	0x0001
#define PV_EOTINT	0x0002
#define PV_EIN0INT	0x0004
#define PV_EIN1INT	0x0008

#define CR_INPINT	0x0010

#define CR_VBINT	0x0080
#define CR_OVBINT	0x0100
#define CR_EVBINT	0x0200
#define CR_AMINT	0x0400
#define CR_TRIGINT	0x0800
#define CR_ADRINT	0x1000
#define CR_AQINT	0x2000

#define CR_VB_FLD	0x4000

#define ICR_MINT	0x00020000
#define ICR_BINT	0x00040000
#define ICR_MAINT	0x00100000
#define ICR_TAINT	0x00200000

/* operation modes */
#define ICP_OP_NO	0x00	/* no valid opmode */
#define ICP_OP_SNAP	0x01	/* snap mode */
#define ICP_OP_GRAB	0x02	/* grab mode */
#define ICP_OP_TRIGGER	0x04	/* multisnap with trigger */
#define ICP_OP_VB_FREEZ 0x08	/* wait for VB in `freeze mode' */
#define ICP_OP_VB_SNAP	0x10	/* wait for VB in `snap mode' */
#define ICP_OP_VB_GRAB	0x20	/* wait for VB in `grab mode' */
#define ICP_OP_OTHER	0x40	/* all other modes */

/* minor device number decoding */
#define ICP_BRD_MASK	0xC0	/* max 4 boards */
#define ICP_BRD_SHIFT	6
#define ICP_MOD_MASK	0x38	/* max 8 acquisition modules */
#define ICP_MOD_SHIFT	3
#define ICP_MOD_IDENT	7	/* self identifying device */
#define ICP_CAM_MASK	0x07	/* max 8 cameras per module */
#define ICP_CAM_SHIFT	0
#define ICP_CAM_IOCTL	7	/* everything does the user */

/* timeout defines */
#define ICP_INIT_TMO	(int)((float)HZ / ICP_INIT_FPS * 1.25)
				/* same value in jiffies */
#define ICP_WAIT_CAMERA	2	/* wait # frames for stabilization
				   of signal */


/* magic sequences (used for test purposes) */
/* should NEVER found in valid (8,16,24bit) images !!?? */
#define ICP_MEM_TEST	0x0F00FFAA
#define ICP_BMA_TEST	0xF0FF0055
     
/*
 * Typedefs:
 */
     
enum _icp_regs_map_t
{
  ICR_REGS,
  CR_REGS,
  NUM_REGS
};  

enum _icp_alloc_mode_t
{
  UNPAGED,
  PAGED
};

enum _icp_sleep_mode_t
{
  CAMERA,
  IOPORT
};

enum icp_error_t
{
  EICP_ERR_NO,
  EICP_MASK,
  EICP_RANGE,
  EICP_NOSUPPORT,
  EICP_POINTER,
  EICP_BUSY
};

struct icp_devno_t
{
  int major;
  int major0;
  int major1;
  int minor;
  int board;
  int module;
  int camera;
};

/* global driver structures */
/* own copy of per board pci config space */
struct icp_pci_t
{
  u_long address0;	/* PCI Interface Control Registers */
  u_long address1;	/* General Control Registers */	
  u_long address2;	/* Acquisition Control Registers */
  u_long address3;	/* On Board Image Memory */
  u_short status;	/* revision level is decoded in status reg */
  u_char revision;	/* board revision number */
  u_char irq_line;	/* used interrupt line */
};

/* mapped regions and their sizes */
struct icp_map_t
{
  u_long *v_icr_base;	/* remapped address0 */
  u_short *v_cr_base;	/* remapped address1 */
  u_short *v_mod_base;	/* remapped address2 */
  int big_mem;		/* flag: 4Mb on board memory (default 2Mb) */
  u_char *v_mem_base;	/* remapped address3 */
  int regs_io;		/* flag: icr/cr regs are not mmaped */
  u_short io_icr_base;	/* address0 is i/o space (replaces v_icr_base) */
  u_short io_cr_base;	/* address1 is i/o space (replaces v_cr_base) */
};

/* per board capabilities */
struct icp_cap_t
{
  int module;		/* type of installed acq. module */
  int cameras;		/* module supports # of cameras */
  int cur_cam;		/* current camera */
  int cam_open;		/* flag: major open lock */
  int amcc0_bug;	/* flag: PCI Interface Chip Rev. 0 bug
			   also means IC-PCI Rev. < A4 */ 
  int amcc1_bug;	/* flag: PCI Interface Chip Rev. 1 bug
			   also means IC-PCI Rev. > A4 < A10 */
  int io_wait8;		/* waiting time between byte access */
  int mem_wait8;	/* loops or microseconds */	
  int io_wait16;	/* waiting time between word access */
  int mem_wait16;	/* loops or microseconds */
  int io_wait32;	/* waiting time between dword access */
  int mem_wait32;	/* loops or microseconds */
  /* AMCC workaround (rev0 bug) */
  int last_operation;
  int latch_status;
  /* AMCC workaround (rev1 bug) */
  int dummy_cnt;
  volatile int *dummy_ptr;
};

/* map of available memory buffers */
struct icp_buf_t
{
  u_long p_buf_base;	/* memory begin (physical addr) */
  void *v_buf_base;	/* memory begin (kernel addr space)*/
  size_t buf_size;	/* overall size of memory */
  off_t   buffer[ICP_BUFFERS_MAX];	/* pointer to start of every buffer */
  int buffers;		/* number of buffers */
  int cur_buf;		/* current buffer */
  int frst_buf;		/* first multisnap buffer */
  int last_buf;		/* last multisnap buffer */
  size_t raw_size;	/* size of buffers in bytes */
  size_t paged_size;	/* size of buffer in bytes (aligned) */
  short	width;		/* horizontal size of buffer */
  short	height;		/* vertical size of buffer */
};


/* global overall structure */
struct icp_global_t
{
  char boards;			/* number of attached boards */
  struct icp_pci_t *pci[ICP_BOARDS_MAX];
  struct icp_map_t *map[ICP_BOARDS_MAX];
  struct icp_cap_t *cap[ICP_BOARDS_MAX];
  struct icp_buf_t *mem[ICP_BOARDS_MAX];
  union	 icp_dev_t *dev[ICP_BOARDS_MAX][ICP_CAMERAS_MAX];
  struct icp_buf_t *bma[ICP_BOARDS_MAX];
  struct icp_acc_t  acc[ICP_BOARDS_MAX];
  struct icp_osp_t *osp[ICP_BOARDS_MAX];
  /* device driver state */
  time_t expire[ICP_BOARDS_MAX];/* timer expiration interval ( HZ value ) */
  int timeout[ICP_BOARDS_MAX];	/* flag: acquisition timeout
				   used in grab/snap mode */
  int opmode[ICP_BOARDS_MAX];	/* operation mode (top / bottom half sync) */
  int complete[ICP_BOARDS_MAX]; /* flag: snap cycle complete
				   used in non-blocking mode */
  char count[ICP_BOARDS_MAX];	/* frame count for multisnap
				   used to determine acq. begin/end */
  int first_read[ICP_BOARDS_MAX];	/* flag: for nonblocking read(2) */ 
  int old_frames[ICP_BOARDS_MAX];	/* frame marker for select (2) */
  int opmode_err[ICP_BOARDS_MAX];	/* flag: opmode for select (2) */
  /* io-port section */
  int read_lock[ICP_BOARDS_MAX];	/* flag: device is already opened
					   in input mode */
  int write_lock[ICP_BOARDS_MAX];	/* flag: device is already opened
					   in output mode */
  char *inp_buffer[ICP_BOARDS_MAX];	/* ptr to kernel space data buffer */
  icp_size_t inp_count[ICP_BOARDS_MAX];	/* remaining bytes to transfer */
  icp_ssize_t inp_index[ICP_BOARDS_MAX];/* bytes already transfered */
  /* global bma stuff */
  int bma_on;		/* flag: use bus-master-mode to transfer
			   data from board- to main- memory */
  u_long p_bma_base;	/* begin of reserved upper memory */
  size_t bma_size;	/* size of reserved upper memory */
  u_char *v_bma_base;	/* remapped reserved upper memory */
  int pieces;		/* number of 2Mb pieces to split memory */
  int weight[ICP_BOARDS_MAX];	/* 2Mb boards 1 / 4Mb boards 2 */
  int errno[ICP_BOARDS_MAX];	/* global error number */

};

  /* ioctl copyin parameters */
union icp_inparam_t
{
  /* generic */
  int set_bmabuf;
  int set_membuf;
  int set_camera;
  union icp_dev_t set_icp_camcnf;
  float	set_fps;
  /* ic-pci */
  int set_snap;
  int set_acqmode;
  int set_trigsrc;
  int set_hvdec;
  /* am-vs */
  struct vs_lut_t set_vs_lut;
  int set_vs_nref;
  int set_vs_pref;
  /* am-dig */
  int set_dg_lutpg;
  int set_dg_olutpg;
  union dg_lut16_t set_dg_lut16;
  struct dg_lut8_t set_dg_lut8;
  struct timespec set_dg_extime;
  /* pcvision */
  struct pv_lut_t set_pv_lut;
  int set_pv_nref;
  int set_pv_pref;

};

  /* ioctl copyout parameters */
union icp_outparam_t
{
  /* generic */
  int get_version;
  int get_module;
  size_t get_bmamapsize;
  int get_bmabuffers;
  int get_bmawidth;
  int get_bmaheight;
  size_t get_bmabufsize;
  int get_bmabuf;
  off_t get_bmabufaddr;
  size_t get_memmapsize;
  int get_membuffers;
  int get_memwidth;
  int get_memheight;
  size_t get_membufsize;
  int get_membuf;
  off_t	get_membufaddr;
  int get_camera;
  union icp_dev_t get_icp_camcnf;
  float	get_fps;
  struct icp_acc_t get_stats;
  /* icp-pci */
  /* am-vs */
  struct vs_lut_t get_vs_lut;
  int get_vs_nref;
  int get_vs_pref;
  /* am-dig */
  int get_dg_lutpg;
  int get_dg_olutpg;
  union dg_lut16_t get_dg_lut16;
  struct dg_lut8_t get_dg_lut8;
  struct timespec get_dg_extime;
  /* pcvision */
  struct pv_lut_t get_pv_lut;
  int get_pv_nref;
  int get_pv_pref;
};

/*
 * Prototypes:
 */

/* icpciMain.c */
void
icp_intr (int);
void
icp_timeout (int);
void
icp_task_queue (int);
int
icp_open (struct icp_devno_t*, icp_flags_t);
int
icp_close (struct icp_devno_t*, icp_flags_t);
icp_ssize_t
icp_read (struct icp_devno_t*, icp_flags_t, char*, icp_size_t);
icp_ssize_t
icp_write (struct icp_devno_t*, icp_flags_t, const char*, icp_size_t);
int
icp_mmap (struct icp_devno_t*, u_long*, size_t);
int
icp_select (struct icp_devno_t*, int);

/* icpciIoctl.c */
int
icp_ioctl (struct icp_devno_t*, icp_flags_t, icp_f_owner_t*, u_int,
	   union icp_inparam_t*, union icp_outparam_t*);

/* icpciAux.c */
int
icp_check_amcc (int, void*, u_char);
void
icp_error(u_char, const char*);
int
icp_decode_devno (struct icp_devno_t*);
int
icp_enable_irq (int, u_long);
int
icp_disable_irq (int, u_long);
int
icp_icr_init (int);
int
icp_cr_init (int);
int
icp_pci_init (int*);
int
icp_map_init (int);
int
icp_cap_init (int);
int
icp_dev_init (int);
void
icp_mem_clear (int, int);
int
icp_mem_copy (int, int, void*);
int
icp_mem_init (int);
void
icp_bma_clear (int, int);
int
icp_bma_copy (int);
int
icp_bma_init (int);

/* icpciAM.c */
int
icp_amvs_startup (int);
int
icp_amvs_init (int, int);
int
icp_amdg_startup (int);
int
icp_amdg_init (int, int);
int
icp_ampv_startup (int);
int
icp_ampv_init (int, int);

/* icpciIOP.c */
int
icp_iop_open (int, icp_flags_t);
int
icp_iop_close (int, icp_flags_t);
icp_ssize_t
icp_iop_read (int, icp_flags_t, char*, icp_size_t);
int
icp_iop_write (int, const char*, icp_size_t);
int
icp_iop_init (int);

#endif /* linux && __KERNEL__ */
#endif /* _ICPCIINT_H */

